#!/usr/bin/python
# -*- coding: koi8-r -*-
import gtk
import gobject

class Interface:
        def __init__(self, progname, stations, drawing_area, \
                     path_cb = None, station_cb = None, city_cb = None, dtime_cb = None, \
                     xsize = 1024, ysize = 800, \
                     citylist = None, citynow = 0):
                if not xsize:
                        xsize = 1024
                if not ysize:
                        ysize = 800
                self.path_cb = path_cb
                self.station_cb = station_cb
                self.city_cb = city_cb
                self.dtime_cb = dtime_cb
                self.stations = stations
                self.dont_set_path = False
                self.dont_set_from = False
                self.dont_set_to = False
                self.citylist = citylist
                self.citynow = citynow
                self.dont_set_city = False
                self.da = drawing_area

                self.win = gtk.Window()
                self.win.set_title("Main")
                self.win.set_wmclass(progname + 'Main', progname)
                self.win.set_resizable(True)
                self.win.set_border_width(5)
                self.xsize = xsize
                self.ysize = ysize
                self.win.set_default_size(self.xsize, self.ysize)
                self.win.connect('configure_event', self.__configure_win_cb)

                vbox = gtk.VBox(False, 5)
                self.win.add(vbox)

                hbox = gtk.HBox(False, 5)
                vbox.pack_start(hbox, False)

                frame = gtk.Frame(_("Left mouse button"))
                hbox.pack_start(frame, True)
                thbox = gtk.HBox(False, 5)
                thbox.set_border_width(5)
                frame.add(thbox)
                label = gtk.Label(_("Source"))
                thbox.pack_start(label, False)
                self.cb_from = self.create_ebox(self.stations)
                self.cb_from.child.connect("changed", self.from_selected)
                thbox.pack_start(self.cb_from, True)

                frame = gtk.Frame(_("Right mouse button"))
                hbox.pack_start(frame, True)
                thbox = gtk.HBox(False, 5)
                thbox.set_border_width(5)
                frame.add(thbox)
                label = gtk.Label(_("Destination"))
                thbox.pack_start(label, False)
                self.cb_to = self.create_ebox(self.stations)
                self.cb_to.child.connect("changed", self.to_selected)
                thbox.pack_start(self.cb_to, True)

                frame = gtk.Frame()
                vbox.pack_start(frame, False)
                tvbox = gtk.VBox(False, 5)
                tvbox.set_border_width(5)
                frame.add(tvbox)

                thbox = gtk.HBox(False, 5)
                tvbox.pack_start(thbox, False)
                label = gtk.Label(_("Founded ways"))
                thbox.pack_start(label, False)
                self.cb_path = gtk.combo_box_new_text()
                thbox.pack_start(self.cb_path, True)
                self.cb_path.set_sensitive(False)
                self.cb_path.connect("changed", self.path_selected)

                thbox = gtk.HBox(False, 5)
                tvbox.pack_start(thbox, False)
                label = gtk.Label(_("Shortest way:"))
                thbox.pack_start(label, False)
                self.label_shortest = gtk.Label("")
                thbox.pack_start(self.label_shortest, False)

                self.cb_city = gtk.combo_box_new_text()
                thbox.pack_end(self.cb_city, False)
                for c in self.citylist:
                        self.cb_city.append_text(c)
                self.cb_city.set_active(self.citynow)
                label = gtk.Label(_("Map:"))
                thbox.pack_end(label, False)
                if len(self.citylist) < 2:
                        self.cb_city.set_sensitive(False)
                self.cb_city.connect("changed", self.city_selected)

                self.cb_dtime = gtk.combo_box_new_text()
                thbox.pack_end(self.cb_dtime, False)
                for c in _('Day'), _('Night'):
                        self.cb_dtime.append_text(c)
                self.cb_dtime.set_active(0)
                label = gtk.Label(_("Time of a day:"))
                thbox.pack_end(label, False)
                self.cb_dtime.connect("changed", self.dtime_selected)

                self.big_vbox = vbox
                self.__pack_da(self.da)

                self.win.show_all()

        def __configure_win_cb(self, widget, event):
                self.xsize, self.ysize = widget.get_size()

        def __pack_da(self, da):
                self.swin = gtk.ScrolledWindow()
                self.swin.set_shadow_type(gtk.SHADOW_IN)
                self.swin.set_policy(gtk.POLICY_ALWAYS, gtk.POLICY_ALWAYS)
                self.big_vbox.pack_start(self.swin, True)
                if da:
                        if type(da) == gtk.DrawingArea:
                                self.swin.add_with_viewport(self.da)
                        else:
                                self.swin.add(self.da)

        def set_da(self, da):
                self.big_vbox.remove(self.swin)
                self.da = da
                self.__pack_da(self.da)
                self.win.show_all()

        def set_path_cb(self, cb):
                self.path_cb = cb

        def set_city_cb(self, cb):
                self.city_cb = cb

        def set_dtime_cb(self, cb):
                self.dtime_cb = cb

        def set_station_cb(self, cb):
                self.station_cb = cb

        def set_dtime(self, dtime):
                self.cb_dtime.set_active(dtime)

        def __num_station__(self, text):
                for s in self.stations:
                        if s[0] == text:
                                return s[1]
                return None

        def __text_station__(self, num):
                for s in self.stations:
                        if s[1] == num:
                                return s[0]
                return ""

        def from_selected(self, foo):
                if not self.dont_set_from:
                        num = self.__num_station__(self.cb_from.child.get_text())
                        if num != None:
                                self.station_cb(num, True)

        def to_selected(self, foo):
                if not self.dont_set_to:
                        num = self.__num_station__(self.cb_to.child.get_text())
                        if num != None:
                                self.station_cb(num, False)

        def set_st_list(self, stations):
                self.stations = stations
                self.dont_set_from = True
                self.dont_set_to = True
                for cb in self.cb_from, self.cb_to:
                        cb.child.set_text("")
                        for p in xrange(len(cb.get_model()), 0, -1):
                                cb.remove_text(p - 1)
                        for s in stations:
                                cb.append_text(s[0])
                self.dont_set_from = False
                self.dont_set_to = False

        def set_from(self, station):
                self.dont_set_from = True
                self.cb_from.child.set_text(self.__text_station__(station))
                self.dont_set_from = False

        def set_to(self, station):
                self.dont_set_to = True
                self.cb_to.child.set_text(self.__text_station__(station))
                self.dont_set_to = False

        def set_path_list(self, plist):
                for p in xrange(len(self.cb_path.get_model()), 0, -1):
                        self.cb_path.remove_text(p - 1)
                for p in plist:
                        self.cb_path.append_text(p)
                if len(plist) > 0:
                        self.label_shortest.set_text(plist[0])
                        self.dont_set_path = True
                        self.cb_path.set_active(0)
                        self.dont_set_path = False
                else:
                        self.label_shortest.set_text("")
                if len(plist) > 1:
                        self.cb_path.set_sensitive(True)
                else:
                        self.cb_path.set_sensitive(False)

        def path_selected(self, foo):
                if self.path_cb and not self.dont_set_path:
                        self.path_cb(self.cb_path.get_active())

        def city_selected(self, foo):
                if self.city_cb and not self.dont_set_city:
                        self.city_cb(self.citylist[self.cb_city.get_active()])

        def dtime_selected(self, foo):
                if self.dtime_cb:
                        self.dtime_cb(self.cb_dtime.get_active())

        def create_ebox(self, names):
                ls = gtk.ListStore(gobject.TYPE_STRING)
                if names:
                        map(lambda a: ls.append([a[0]]), names)
                ce = gtk.ComboBoxEntry(ls, 0)
                compl = gtk.EntryCompletion()
                compl.set_model(ls)
                compl.set_text_column(0)
                ce.child.set_completion(compl)
                return ce
