#!/usr/bin/python
# -*- coding: koi8-r -*-
import gtk
import gobject

white = gtk.gdk.Color(red = 65535, green = 65535, blue = 65535)
black = gtk.gdk.Color(red = 0, green = 0, blue = 0)
red = gtk.gdk.Color(red = 65535, green = 0, blue = 0)
green = gtk.gdk.Color(red = 0, green = 65535, blue = 0)
blue = gtk.gdk.Color(red = 0, green = 0, blue = 65535)
yellow = gtk.gdk.Color(red = 65535, green = 65535, blue = 0)

class MapDisplay:
        def __init__(self, coords, cadd, linecoords, selcb = None, img = None):
                self.gc = None
                self.scb = selcb
                self.coords = coords
                self.cadd = cadd
                self.pic = gtk.gdk.pixbuf_new_from_file(img)
                self.linecoords = linecoords

                self.da = gtk.DrawingArea()
                self.da.set_size_request(self.pic.get_width(), self.pic.get_height())
                self.da.set_events(gtk.gdk.EXPOSURE_MASK | gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.KEY_PRESS_MASK | gtk.gdk.POINTER_MOTION_MASK)
                self.da.connect("expose_event", self.expose)
                self.da.connect("button_press_event", self.bp)
                self.da.connect('configure_event', self.__configure_cb)

                self.daw = self.da.get_allocation().width
                self.dah = self.da.get_allocation().height
                self.pw = self.pic.get_width()
                self.ph = self.pic.get_height()
                self.dx = 0
                self.dy = 0

                self.stfrom = None
                self.stto = None
                self.pathlist = None
                self.drawpath = False
                self.pathwidth = 4

                self.timeout = gobject.timeout_add(400, self.redraw, False)
                self.step = 0

                self.STATE_OFF  = 0
                self.STATE_FROM = 1
                self.STATE_TO   = 2

        def __draw_dash_rect(self, step, c1, c2, x, y, w, h):
                cs = c1
                ce = c2
                if step == 0:
                        cs = c2
                        ce = c1
                self.gc.set_rgb_fg_color(cs)
                self.gc.set_line_attributes(2, gtk.gdk.LINE_SOLID, 0, 0)
                self.da.window.draw_rectangle(self.gc, False, x, y, w, h)
                self.gc.set_rgb_fg_color(ce)
                self.gc.set_line_attributes(2, gtk.gdk.LINE_ON_OFF_DASH, 0, 0)
                self.da.window.draw_rectangle(self.gc, False, x, y, w, h)

        def __draw_dash_arc(self, step, c1, c2, x, y, d):
                cs = c1
                ce = c2
                if step == 0:
                        cs = c2
                        ce = c1
                self.gc.set_rgb_fg_color(cs)
                self.gc.set_line_attributes(2, gtk.gdk.LINE_SOLID, 0, 0)
                self.da.window.draw_arc(self.gc, False, x, y, d, d, 0, 64 * 360)
                self.gc.set_rgb_fg_color(ce)
                self.gc.set_line_attributes(2, gtk.gdk.LINE_ON_OFF_DASH, 0, 0)
                self.da.window.draw_arc(self.gc, False, x, y, d, d, 0, 64 * 360)

        def draw_station(self, num, state):
                if num == None:
                        return

                sx, sy, sd = self.coords[num][0][0], self.coords[num][0][1], self.coords[num][1] / 2
                snx, sny, snw, snh = self.coords[num][2]
                snx -= 2
                sny -= 2
                snw += 4
                snh += 4

                if state == self.STATE_OFF:
                        self.draw_pic(sx - sd - 1, sy - sd - 1, sd * 2 + 2, sd * 2 + 2)
                        self.draw_pic(snx - 1, sny  - 1, snw + 2, snh + 2)
                else:
                        c = blue
                        if state == self.STATE_FROM:
                                c = red
                        self.__draw_dash_rect(self.step, c, white, snx + self.dx, sny + self.dy, snw, snh)
                        self.__draw_dash_arc(self.step, c, white, self.dx + sx - sd, self.dy + sy - sd, sd * 2)

        def draw_path(self, path, state):
                points = []
                for n in xrange(len(path) - 1):
                        points.append((self.coords[path[n]][0][0], self.coords[path[n]][0][1]))
                        if self.cadd.has_key((path[n], path[n + 1])):
                                ca = self.cadd[(path[n], path[n + 1])]
                                for i in xrange((len(ca) - 1) / 2):
                                        points.append((ca[1 + i * 2], ca[2 + i * 2]))
                        points.append((self.coords[path[n + 1]][0][0], self.coords[path[n + 1]][0][1]))

                if state:
                        points = map(lambda a: (a[0] + self.dx, a[1] + self.dy), points)

                        self.gc.set_line_attributes(self.pathwidth + 2, gtk.gdk.LINE_SOLID, gtk.gdk.CAP_ROUND, 0)
                        self.gc.set_rgb_fg_color(black)
                        self.da.window.draw_lines(self.gc, points)
                        self.gc.set_line_attributes(self.pathwidth, gtk.gdk.LINE_ON_OFF_DASH, gtk.gdk.CAP_BUTT, 0)
                        self.gc.set_rgb_fg_color(white)
                        self.da.window.draw_lines(self.gc, points)
                else:
                        minx = max(min(map(lambda a: a[0], points)) - self.pathwidth - 2, 0)
                        miny = max(min(map(lambda a: a[1], points)) - self.pathwidth - 2, 0)
                        maxx = min(max(map(lambda a: a[0], points)) + self.pathwidth + 2, self.pw - 1)
                        maxy = min(max(map(lambda a: a[1], points)) + self.pathwidth + 2, self.ph - 1)
                        self.draw_pic(minx, miny, maxx - minx, maxy - miny)

        def redraw(self, full):
                if not self.da.window:
                        return False
                if not self.gc:
                        return True
                if full and self.pathlist and len(self.pathlist):
                        self.draw_path(self.pathlist, self.drawpath)
                self.step = 1 - self.step
                self.draw_station(self.stfrom, self.STATE_FROM)
                self.draw_station(self.stto, self.STATE_TO)
                return True

        def set_path_list(self, pathlist):
                if self.pathlist and len(self.pathlist):
                        self.drawpath = False
                        self.redraw(True)
                self.pathlist = pathlist
                self.drawpath = True
                self.redraw(True)

        def set_start_station(self, st):
                self.draw_station(self.stfrom, self.STATE_OFF)
                self.stfrom = st
                self.redraw(True)

        def set_stop_station(self, st):
                self.draw_station(self.stto, self.STATE_OFF)
                self.stto = st
                self.redraw(True)

        def set_selection_cb(self, cb):
                self.scb = cb

        def bp(self, widget, event):
                if event.button != 1 and event.button != 3:
                        return

                start = False
                if event.button == 1:
                        start = True

                x = event.get_coords()[0] - self.dx
                y = event.get_coords()[1] - self.dy

                for st in self.coords.keys():
                        sx, sy, sd = self.coords[st][0][0], self.coords[st][0][1], self.coords[st][1] / 2
                        snx, sny, snw, snh = self.coords[st][2]
                        if (x > sx - sd and x < sx + sd and y > sy - sd and y < sy + sd) or\
                           (x > snx and x < snx + snw and y > sny and y < sny + snh):
                                if st != self.stto and st != self.stfrom and self.scb:
                                        self.scb(st, start)
                                break

        def __update_vars(self):
                self.daw = self.da.get_allocation().width
                self.dah = self.da.get_allocation().height
                self.pw = self.pic.get_width()
                self.ph = self.pic.get_height()
                self.dx = (self.daw - self.pw) / 2
                self.dy = (self.dah - self.ph) / 2

        def __configure_cb(self, foo, bar):
                self.__update_vars()

        def expose(self, widget, event):
                if not self.gc:
                        self.gc = gtk.gdk.GC(self.da.window)
                        self.gc.set_fill(gtk.gdk.SOLID)

                x, y, w, h = event.area.x, event.area.y, event.area.width, event.area.height

                #clip event region
                if x < self.dx:
                        w -= self.dx - x
                        x = self.dx
                if y < self.dy:
                        h -= self.dy - y
                        y = self.dy

                if x + w > self.dx + self.pw:
                        w = self.pw + self.dx - x

                if y + h > self.dy + self.ph:
                        h = self.ph + self.dy - y

                if w > 0 and h > 0:
			self.da.window.draw_pixbuf(self.gc, self.pic, x - self.dx, y - self.dy, x, y, w, h, 0, 0)
                        self.redraw(True)

        def draw_pic(self, x, y, w, h):
		self.da.window.draw_pixbuf(self.gc, self.pic, x, y, self.dx + x, self.dy + y, w, h, 0, 0)

