#!/usr/bin/python
# -*- coding: koi8-r -*-
import gtk
import gobject
import gnome.canvas

class MapDisplay:
        def __init__(self, coords, cadd, linecoords, selcb = None, img = None):
                self.gc = None
                self.scb = selcb
                self.coords = coords
                self.cadd = cadd

                self.da = gnome.canvas.Canvas(aa = True)
                self.xs = min(map(lambda a: a[2][0] - 10, self.coords.values()) + \
                              map(lambda a: a[0][0] - a[1] - 5, self.coords.values()))
                self.ys = min(map(lambda a: a[2][1] - 10, self.coords.values()) + \
                              map(lambda a: a[0][1] - a[1] - 5, self.coords.values()))
                self.dw = max(map(lambda a: a[2][0] + a[2][2] + 10, self.coords.values()) + \
                              map(lambda a: a[0][0] + a[1] + 5, self.coords.values()))
                self.dh = max(map(lambda a: a[2][1] + a[2][3] + 10, self.coords.values()) + \
                              map(lambda a: a[0][1] + a[1] + 5, self.coords.values()))

                self.da.set_scroll_region(self.xs - 20, self.ys - 20, self.dw + 20, self.dh + 20)
                self.root = self.da.root()

                self.root.add(gnome.canvas.CanvasRect, outline_color = "black", fill_color = "white",
                              x1 = self.xs - 20, y1 = self.ys - 20, x2 = self.dw + 20, y2 = self.dh + 20)

                self.linecoords = linecoords
                self.bpath = dict()
                self.stations = dict()

                for k in self.linecoords.keys():
                        lc = self.linecoords[k]['color']
                        c = "#%02x%02x%02x" % (max(int(lc[0:2], 16) - 50, 0),
                                               max(int(lc[2:4], 16) - 50, 0),
                                               max(int(lc[4:6], 16) - 50, 0))
                        self.bpath[k] = self.root.add(gnome.canvas.CanvasBpath,
                                                      outline_color = c,
                                                      width_pixels = 12, cap_style = gtk.gdk.CAP_NOT_LAST)
                        c = "#%02x%02x%02x" % (max(int(lc[0:2], 16) - 20, 0),
                                               max(int(lc[2:4], 16) - 20, 0),
                                               max(int(lc[4:6], 16) - 20, 0))
                        self.bpath[(k, 1)] = self.root.add(gnome.canvas.CanvasBpath,
                                                           outline_color = c,
                                                           width_pixels = 10, cap_style = gtk.gdk.CAP_NOT_LAST)
                        c = "#%s" % lc
                        self.bpath[(k, 2)] = self.root.add(gnome.canvas.CanvasBpath,
                                                           outline_color = c,
                                                           width_pixels = 6, cap_style = gtk.gdk.CAP_NOT_LAST)
                        n = self.linecoords[k]['stations']
                        path = []
                        path.append((gnome.canvas.MOVETO, n[0][1][0], n[0][1][1]))
                        for i in xrange(len(n) - 1):
                                spline = False
                                np = [(n[i][1][0], n[i][1][1]), (n[i + 1][1][0], n[i + 1][1][1])]
                                if self.cadd.has_key((n[i][0], n[i + 1][0])):
                                        ca = self.cadd[n[i][0], n[i + 1][0]]
                                        for ni in xrange((len(ca) - 1) / 2):
                                                np.insert(-1, (ca[1 + ni * 2], ca[2 + ni * 2]))
                                        spline = ca[0]
                                if spline: #spline
                                        self.create_splinepath(path, np)
                                else: #line
                                        for p in np:
                                                path.append((gnome.canvas.LINETO, p[0], p[1]))

                        path.append((gnome.canvas.MOVETO, 0, 0))
                        path_def = gnome.canvas.path_def_new(path)
                        self.bpath[k].set_bpath(path_def)
                        self.bpath[(k, 1)].set_bpath(path_def)
                        self.bpath[(k, 2)].set_bpath(path_def)

                for k in self.linecoords.keys():
                        n = self.linecoords[k]['stations']
                        for i in xrange(len(n)):
                                r = self.linecoords[k]['diameter'] / 2
                                self.stations[(k, i)] = self.root.add(gnome.canvas.CanvasEllipse,
                                                                      outline_color = "yellow",
                                                                      fill_color = "#" + self.linecoords[k]['color'],
                                                                      width_pixels = 1,
                                                                      x1 = n[i][1][0] - r, y1 = n[i][1][1] - r,
                                                                      x2 = n[i][1][0] + r, y2 = n[i][1][1] + r)
                                #self.root.add(gnome.canvas.CanvasText, text = str(i),
                                #              fill_color = "white", x = n[i][1][0], y = n[i][1][1],
                                #              font = "Arial, 10")

        def set_selection_cb(self, cb):
                pass

        def find_cross(self, lx1, ly1, lx2, ly2, x, y):
                a = lx2 - lx1
                b = ly2 - ly1
                c = a * x + b * y
                d = b
                e = -a
                f = d * lx1 + e * ly1
                g = a * e - b * d
                return float((c * e - b * f) / g), float((a * f - c * d) / g)

        def create_splinepath(self, path, points):
                guide = []
                div = 3
                prevcx, prevcy = points[0]
                for i in xrange(len(points) - 1):
                        if i == len(points) - 2:
                                cx, cy = points[i + 1][0], points[i + 1][1]
                        else:
                                #FIXME: bad method for interpolation =/
                                nx, ny = self.find_cross(points[i][0], points[i][1],
                                                         points[i + 2][0], points[i + 2][1],
                                                         points[i + 1][0], points[i + 1][1])
                                ax = points[i][0] - (nx - points[i + 1][0])
                                ay = points[i][1] - (ny - points[i + 1][1])
                                bx = points[i + 2][0] - (nx - points[i + 1][0])
                                by = points[i + 2][1] - (ny - points[i + 1][1])

                                if abs(points[i][0] - points[i + 1][0]) > 40:
                                        div = 2
                                else:
                                        div = 3

                                ex = nx - float((nx - points[i][0]) / div)
                                ey = ny - float((ny - points[i][1]) / div)
                                fx = nx + float((points[i + 2][0] - nx) / div)
                                fy = ny + float((points[i + 2][1] - ny) / div)

                                cx, cy = self.find_cross(bx, by, ax, ay, ex, ey)
                                npcx, npcy = self.find_cross(bx, by, ax, ay, fx, fy)

                        path.append((gnome.canvas.CURVETO, prevcx, prevcy, cx, cy,
                                                           points[i + 1][0], points[i + 1][1]))
                        prevcx, prevcy = npcx, npcy
